library(readxl)
library(phylolm)
library(future)

# Set working directory or create project folder before running code

mammals.HTF <- read_excel("HTF_Mammals.xlsx", sheet = "HTF_Mammals") # all datasets are formatted identically, replace names (e.g., "HTF_Birds.xlsx", sheet = "HTF_Birds") to analyse other taxonomic groups

# Create dataframe containing variables for analyses, see README file for description of each variable, replace Species_MammalTree with Species_BirdTree, etc depending on dataset being analysed
mammals <- data.frame(species=mammals.HTF$Species_MammalTree, order=mammals.HTF$Order, family=mammals.HTF$Family,
                      rl.cat=mammals.HTF$Red_List_Category, pop.trend=mammals.HTF$Population_Trend2,
                      realm=mammals.HTF$Realm2, gr.1realm=mammals.HTF$`Occurs_In_>1_Realm`,
                      tfspecies=as.numeric(mammals.HTF$'1_Tropical_Forest'), htf.dependency=mammals.HTF$HTF_Dependency, criterionB=mammals.HTF$Criterion_B, criterionAB=mammals.HTF$Criterion_AB,
                      d.forcov.allr=scale(mammals.HTF$Rel_Diff_Forest_NonForest_AllRealms), d.fsci.allr=scale(mammals.HTF$Rel_Diff_H_L_FSCI_AllRealms), d.fsii.allr=scale(mammals.HTF$Rel_Diff_H_L_FSII_AllRealms),
                      d.forcov.h.allr=scale(mammals.HTF$Rel_Diff_Forest_NonForest_H_AllRealms), d.fsci.h.allr=scale(mammals.HTF$Rel_Diff_H_L_FSCI_H_AllRealms), d.fsii.h.allr=scale(mammals.HTF$Rel_Diff_H_L_FSII_H_AllRealms),
                      d.forcov.l.allr=scale(mammals.HTF$Rel_Diff_Forest_NonForest_L_AllRealms), d.fsci.l.allr=scale(mammals.HTF$Rel_Diff_H_L_FSCI_L_AllRealms), d.fsii.l.allr=scale(mammals.HTF$Rel_Diff_H_L_FSII_L_AllRealms),
                      d.hfp.allr=scale(mammals.HTF$Rel_Diff_H_L_HFP_AllRealms),
                      d.fsci.hm.allr=scale(mammals.HTF$Rel_Diff_HM_L_FSCI_AllRealms), d.fsii.hm.allr=scale(mammals.HTF$Rel_Diff_HM_L_FSII_AllRealms),
                      prop.forcov.c=scale(mammals.HTF$Proportion_Loss), prop.fsci.c=scale(mammals.HTF$Proportion_FSCI_C_AllRealms))

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Red List or threatened status as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

#----------------------------------#
# Recode Red List status as numeric
#----------------------------------#
# Main: Threatened = CR/EN/VU = 1, Not threatened = NT/LC = 0
mammals[,4] <- ifelse(mammals[,4] == "Critically Endangered", 1,
                      ifelse(mammals[,4] == "Endangered", 1,
                             ifelse(mammals[,4] == "Vulnerable", 1,
                                    ifelse(mammals[,4] == "Near Threatened", 0,
                                           ifelse(mammals[,4] == "Least Concern", 0,
                                                  ifelse(mammals[,4] == "Data Deficient", 2, NA))))))

mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests

# These two lines allow subsetting of species by htf dependency, use one at a time depending on whether rainforest obligate or associated species are being analysed
mammals5 <- subset(mammals4, htf.dependency == "Exclusive") # keep only humid tropical forest obligate species 
#mammals5 <- subset(mammals4, htf.dependency == "Optional") # keep only humid tropical forest associated species

mammals6 <- subset(mammals5, rl.cat < 2) # remove Data Deficient or DD species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# use these lines when removing species that are listed in IUCN Red List Critera A and B, will need to find and replace mammals6 in the model code below with mammals7
#mammals7 <- subset(mammals6, criterionAB == "N") # remove criterion B or AB species
#rownames(mammals7) <- mammals7$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species, replace mammaltree with birdtree, etc. depending on dataset being analysed
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#----------------------------------------------------------------------------#
# Forest Cover + FSCI + FSII: multiple phylogenetic logistic regression model
#----------------------------------------------------------------------------#
# loops through each of 100 trees selected randonly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  # 4 models specified here, run one at a time with the others commented out. Each model has a different specification of the predictor variable - see comment beside each model
  cover_fsci_fsii <- phyloglm(rl.cat ~ d.forcov.allr + d.fsci.allr + d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII
  #cover_fsci_fsii <- phyloglm(rl.cat ~ d.forcov.l.allr + d.fsci.l.allr + d.fsii.l.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, based on simulated underestimate of high SCI and FSII
  #cover_fsci_fsii <- phyloglm(rl.cat ~ d.forcov.h.allr + d.fsci.h.allr + d.fsii.h.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, based on simulated overestimate of high SCI and FSII
  #cover_fsci_fsii <- phyloglm(rl.cat ~ d.forcov.allr + d.fsci.hm.allr + d.fsii.hm.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, where high and moderate condition/integrity forests are pooled
  
  # Estimated parameters to extract from models at each iteration
  # model selection
  aic.cover_fsci_fsii <- cover_fsci_fsii$aic # AIC
  logl.cover_fsci_fsii <- cover_fsci_fsii$logLik # log likelihood
  plogl.cover_fsci_fsii <- cover_fsci_fsii$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.cover_fsci_fsii, logl.cover_fsci_fsii, plogl.cover_fsci_fsii) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover_fsci_fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_fsci_fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_fsci_fsii <- cover_fsci_fsii$X
  yvalues.cover_fsci_fsii <- cover_fsci_fsii$y
  lin.pred.cover_fsci_fsii <- cover_fsci_fsii$linear.predictors
  fitted.val.cover_fsci_fsii <- cover_fsci_fsii$fitted.values
  residuals.cover_fsci_fsii <- cover_fsci_fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_fsci_fsii, yvalues.cover_fsci_fsii, lin.pred.cover_fsci_fsii, fitted.val.cover_fsci_fsii, residuals.cover_fsci_fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/cover_fsci_fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef.confint, file = "Mammals/RL/cover_fsci_fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha.confint, file = "Mammals/RL/cover_fsci_fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/RL/cover_fsci_fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop

#---------------------------------------------------------#
# FSII: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsii <- phyloglm(rl.cat ~ d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsii <- fsii$aic
  logl.fsii <- fsii$logLik
  plogl.fsii <- fsii$penlogLik
  mod.sel <- cbind(aic.fsii, logl.fsii, plogl.fsii)
  
  # coefficients
  s <- summary(fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsii <- fsii$X
  yvalues.fsii <- fsii$y
  lin.pred.fsii <- fsii$linear.predictors
  fitted.val.fsii <- fsii$fitted.values
  residuals.fsii <- fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsii, yvalues.fsii, lin.pred.fsii, fitted.val.fsii, residuals.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#---------------------------------------------------------#
# FSCI: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsci <- phyloglm(rl.cat ~ d.fsci.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsci <- fsci$aic
  logl.fsci <- fsci$logLik
  plogl.fsci <- fsci$penlogLik
  mod.sel <- cbind(aic.fsci, logl.fsci, plogl.fsci)
  
  # coefficients
  s <- summary(fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsci <- fsci$X
  yvalues.fsci <- fsci$y
  lin.pred.fsci <- fsci$linear.predictors
  fitted.val.fsci <- fsci$fitted.values
  residuals.fsci <- fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsci, yvalues.fsci, lin.pred.fsci, fitted.val.fsci, residuals.fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#---------------------------------------------------------#
# Cover: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover <- phyloglm(rl.cat ~ d.forcov.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover <- cover$aic
  logl.cover <- cover$logLik
  plogl.cover <- cover$penlogLik
  mod.sel <- cbind(aic.cover, logl.cover, plogl.cover)
  
  # coefficients
  s <- summary(cover)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover <- cover$X
  yvalues.cover <- cover$y
  lin.pred.cover <- cover$linear.predictors
  fitted.val.cover <- cover$fitted.values
  residuals.cover <- cover$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover, yvalues.cover, lin.pred.cover, fitted.val.cover, residuals.cover)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/cover_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/cover_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/cover_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/cover_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------------------------#
# HFP: univariate phylogenetic logistic regression model
#--------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  hfp <- phyloglm(rl.cat ~ d.hfp.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.hfp <- hfp$aic
  logl.hfp <- hfp$logLik
  plogl.hfp <- hfp$penlogLik
  mod.sel <- cbind(aic.hfp, logl.hfp, plogl.hfp)
  
  # coefficients
  s <- summary(hfp)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(hfp), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.hfp <- hfp$X
  yvalues.hfp <- hfp$y
  lin.pred.hfp <- hfp$linear.predictors
  fitted.val.hfp <- hfp$fitted.values
  residuals.hfp <- hfp$residuals
  all.sp.specific <- data.frame(species.sp.coef.hfp, yvalues.hfp, lin.pred.hfp, fitted.val.hfp, residuals.hfp)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/hfp_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/hfp_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/hfp_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/hfp_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#----------------------------------------------------------------------------#
# Cover + HFP + FSCI + FSII: multiple phylogenetic logistic regression model
#----------------------------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover_hfp_fsci_fsii <- phyloglm(rl.cat ~ d.forcov.allr + d.hfp.allr + d.fsci.allr + d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$aic
  logl.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$logLik
  plogl.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$penlogLik
  mod.sel <- cbind(aic.cover_hfp_fsci_fsii, logl.cover_hfp_fsci_fsii, plogl.cover_hfp_fsci_fsii)
  
  # coefficients
  s <- summary(cover_hfp_fsci_fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_hfp_fsci_fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:5], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:5], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,6], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,6], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$X
  yvalues.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$y
  lin.pred.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$linear.predictors
  fitted.val.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$fitted.values
  residuals.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_hfp_fsci_fsii, yvalues.cover_hfp_fsci_fsii, lin.pred.cover_hfp_fsci_fsii, fitted.val.cover_hfp_fsci_fsii, residuals.cover_hfp_fsci_fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/cover_hfp_fsci_fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/cover_hfp_fsci_fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/cover_hfp_fsci_fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/cover_hfp_fsci_fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------#
# Change in Cover + Change in FSCI
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover_fsci <- phyloglm(rl.cat ~ prop.forcov.c + prop.fsci.c, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover_fsci <- cover_fsci$aic
  logl.cover_fsci <- cover_fsci$logLik
  plogl.cover_fsci <- cover_fsci$penlogLik
  mod.sel <- cbind(aic.cover_fsci, logl.cover_fsci, plogl.cover_fsci)
  
  # coefficients
  s <- summary(cover_fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:3], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:3], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,4], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,4], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_fsci <- cover_fsci$X
  yvalues.cover_fsci <- cover_fsci$y
  lin.pred.cover_fsci <- cover_fsci$linear.predictors
  fitted.val.cover_fsci <- cover_fsci$fitted.values
  residuals.cover_fsci <- cover_fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_fsci, yvalues.cover_fsci, lin.pred.cover_fsci, fitted.val.cover_fsci, residuals.cover_fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/change_cover_fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/change_cover_fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/change_cover_fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/change_cover_fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Population Trend as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

# Read in the Excel workbook with the data as above before running the lines below

#-----------------------------------#
# Recode Population Trend as numeric
#-----------------------------------#
mammals[,5] <- ifelse(mammals[,5] == "Decreasing", 1,
                      ifelse(mammals[,5] == "Stable or Increasing", 0,
                             ifelse(mammals[,5] == "Unknown", 2, NA)))


mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests

# These two lines allow subsetting of species by htf dependency, use one at a time depending on whether rainforest obligate or associated species are being analysed
mammals5 <- subset(mammals4, htf.dependency == "Exclusive") # keep only humid tropical forest obligate species 
#mammals5 <- subset(mammals4, htf.dependency == "Optional") # keep only humid tropical forest associated species

mammals6 <- subset(mammals5, pop.trend < 2) # remove unknown population trend species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#----------------------------------------------------------------------------#
# Forest Cover + FSCI + FSII: multiple phylogenetic logistic regression model
#----------------------------------------------------------------------------#
# loops through each of 100 trees selected randomly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  # 4 models specified here, run one at a time with the others commented out. Each model has a different specification of the predictor variable - see comment beside each model
  cover_fsci_fsii <- phyloglm(pop.trend ~ d.forcov.allr + d.fsci.allr + d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII
  #cover_fsci_fsii <- phyloglm(pop.trend ~ d.forcov.l.allr + d.fsci.l.allr + d.fsii.l.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, based on simulated underestimate of high SCI and FSII
  #cover_fsci_fsii <- phyloglm(pop.trend ~ d.forcov.h.allr + d.fsci.h.allr + d.fsii.h.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, based on simulated overestimate of high SCI and FSII
  #cover_fsci_fsii <- phyloglm(pop.trend ~ d.forcov.allr + d.fsci.hm.allr + d.fsii.hm.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000) # forest cover + FSCI + FSII, where high and moderate condition/integrity forests are pooled
  
  # Estimated parameters to extract from models at each iteration
  # model selection
  aic.cover_fsci_fsii <- cover_fsci_fsii$aic # AIC
  logl.cover_fsci_fsii <- cover_fsci_fsii$logLik # log likelihood
  plogl.cover_fsci_fsii <- cover_fsci_fsii$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.cover_fsci_fsii, logl.cover_fsci_fsii, plogl.cover_fsci_fsii) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover_fsci_fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_fsci_fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_fsci_fsii <- cover_fsci_fsii$X
  yvalues.cover_fsci_fsii <- cover_fsci_fsii$y
  lin.pred.cover_fsci_fsii <- cover_fsci_fsii$linear.predictors
  fitted.val.cover_fsci_fsii <- cover_fsci_fsii$fitted.values
  residuals.cover_fsci_fsii <- cover_fsci_fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_fsci_fsii, yvalues.cover_fsci_fsii, lin.pred.cover_fsci_fsii, fitted.val.cover_fsci_fsii, residuals.cover_fsci_fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/cover_fsci_fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef.confint, file = "Mammals/PT/cover_fsci_fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha.confint, file = "Mammals/PT/cover_fsci_fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/PT/cover_fsci_fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop

#---------------------------------------------------------#
# FSII: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsii <- phyloglm(pop.trend ~ d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsii <- fsii$aic
  logl.fsii <- fsii$logLik
  plogl.fsii <- fsii$penlogLik
  mod.sel <- cbind(aic.fsii, logl.fsii, plogl.fsii)
  
  # coefficients
  s <- summary(fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsii <- fsii$X
  yvalues.fsii <- fsii$y
  lin.pred.fsii <- fsii$linear.predictors
  fitted.val.fsii <- fsii$fitted.values
  residuals.fsii <- fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsii, yvalues.fsii, lin.pred.fsii, fitted.val.fsii, residuals.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#---------------------------------------------------------#
# FSCI: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsci <- phyloglm(pop.trend ~ d.fsci.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsci <- fsci$aic
  logl.fsci <- fsci$logLik
  plogl.fsci <- fsci$penlogLik
  mod.sel <- cbind(aic.fsci, logl.fsci, plogl.fsci)
  
  # coefficients
  s <- summary(fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsci <- fsci$X
  yvalues.fsci <- fsci$y
  lin.pred.fsci <- fsci$linear.predictors
  fitted.val.fsci <- fsci$fitted.values
  residuals.fsci <- fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsci, yvalues.fsci, lin.pred.fsci, fitted.val.fsci, residuals.fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#---------------------------------------------------------#
# Cover: univariate phylogenetic logistic regression model
#---------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover <- phyloglm(pop.trend ~ d.forcov.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover <- cover$aic
  logl.cover <- cover$logLik
  plogl.cover <- cover$penlogLik
  mod.sel <- cbind(aic.cover, logl.cover, plogl.cover)
  
  # coefficients
  s <- summary(cover)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover <- cover$X
  yvalues.cover <- cover$y
  lin.pred.cover <- cover$linear.predictors
  fitted.val.cover <- cover$fitted.values
  residuals.cover <- cover$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover, yvalues.cover, lin.pred.cover, fitted.val.cover, residuals.cover)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/cover_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/cover_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/cover_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/cover_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------------------------#
# HFP: univariate phylogenetic logistic regression model
#--------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  hfp <- phyloglm(pop.trend ~ d.hfp.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.hfp <- hfp$aic
  logl.hfp <- hfp$logLik
  plogl.hfp <- hfp$penlogLik
  mod.sel <- cbind(aic.hfp, logl.hfp, plogl.hfp)
  
  # coefficients
  s <- summary(hfp)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(hfp), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:2], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:2], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,3], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,3], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.hfp <- hfp$X
  yvalues.hfp <- hfp$y
  lin.pred.hfp <- hfp$linear.predictors
  fitted.val.hfp <- hfp$fitted.values
  residuals.hfp <- hfp$residuals
  all.sp.specific <- data.frame(species.sp.coef.hfp, yvalues.hfp, lin.pred.hfp, fitted.val.hfp, residuals.hfp)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/hfp_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/hfp_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/hfp_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/hfp_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#----------------------------------------------------------------------------#
# Cover + HFP + FSCI + FSII: multiple phylogenetic logistic regression model
#----------------------------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover_hfp_fsci_fsii <- phyloglm(pop.trend ~ d.forcov.allr + d.hfp.allr + d.fsci.allr + d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=40, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$aic
  logl.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$logLik
  plogl.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$penlogLik
  mod.sel <- cbind(aic.cover_hfp_fsci_fsii, logl.cover_hfp_fsci_fsii, plogl.cover_hfp_fsci_fsii)
  
  # coefficients
  s <- summary(cover_hfp_fsci_fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_hfp_fsci_fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:5], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:5], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,6], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,6], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$X
  yvalues.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$y
  lin.pred.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$linear.predictors
  fitted.val.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$fitted.values
  residuals.cover_hfp_fsci_fsii <- cover_hfp_fsci_fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_hfp_fsci_fsii, yvalues.cover_hfp_fsci_fsii, lin.pred.cover_hfp_fsci_fsii, fitted.val.cover_hfp_fsci_fsii, residuals.cover_hfp_fsci_fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/cover_hfp_fsci_fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/cover_hfp_fsci_fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/cover_hfp_fsci_fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/cover_hfp_fsci_fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------#
# Change in Cover + Change in FSCI
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover_fsci <- phyloglm(pop.trend ~ prop.forcov.c + prop.fsci.c, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover_fsci <- cover_fsci$aic
  logl.cover_fsci <- cover_fsci$logLik
  plogl.cover_fsci <- cover_fsci$penlogLik
  mod.sel <- cbind(aic.cover_fsci, logl.cover_fsci, plogl.cover_fsci)
  
  # coefficients
  s <- summary(cover_fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover_fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:3], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:3], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,4], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,4], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover_fsci <- cover_fsci$X
  yvalues.cover_fsci <- cover_fsci$y
  lin.pred.cover_fsci <- cover_fsci$linear.predictors
  fitted.val.cover_fsci <- cover_fsci$fitted.values
  residuals.cover_fsci <- cover_fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover_fsci, yvalues.cover_fsci, lin.pred.cover_fsci, fitted.val.cover_fsci, residuals.cover_fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/change_cover_fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/change_cover_fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/change_cover_fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/change_cover_fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop
