library(readxl)
library(phylolm)
library(future)

# Set working directory or create project folder before running code

mammals.HTF <- read_excel("HTF_Mammals.xlsx", sheet = "HTF_Mammals") # all datasets are formatted identically, replace names (e.g., "HTF_Birds.xlsx", sheet = "HTF_Birds") to analyse other taxonomic groups

# Create dataframe containing variables for analyses, see README file for description of each variable, replace Species_MammalTree with Species_BirdTree, etc depending on dataset being analysed
mammals <- data.frame(species=mammals.HTF$Species_MammalTree, order=mammals.HTF$Order, family=mammals.HTF$Family,
                      rl.cat=mammals.HTF$Red_List_Category2, pop.trend=mammals.HTF$Population_Trend2,
                      realm=mammals.HTF$Realm2, gr.1realm=mammals.HTF$`Occurs_In_>1_Realm`,
                      tfspecies=as.numeric(mammals.HTF$'1_Tropical_Forest'), htf.dependency=mammals.HTF$HTF_Dependency,
                      d.forcov.allr=scale(mammals.HTF$Rel_Diff_Forest_NonForest_AllRealms), d.fsci.allr=scale(mammals.HTF$Rel_Diff_H_L_FSCI_AllRealms), d.fsii.allr=scale(mammals.HTF$Rel_Diff_H_L_FSII_AllRealms))

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Red List or threatened status as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

#----------------------------------#
# Recode Red List status as numeric
#----------------------------------#
mammals[,4] <- ifelse(mammals[,4] == "Threatened", 1,
                      ifelse(mammals[,4] == "Not Threatened", 0,
                             ifelse(mammals[,4] == "Data Deficient", 2, NA)))

mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests

# These two lines allow subsetting of species by htf dependency, use one at a time depending on whether rainforest obligate or associated species are being analysed
mammals5 <- subset(mammals4, htf.dependency == "Exclusive") # keep only humid tropical forest obligate species 
#mammals5 <- subset(mammals4, htf.dependency == "Optional") # keep only humid tropical forest associated species

mammals6 <- subset(mammals5, rl.cat < 2) # remove Data Deficient or DD species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species, replace mammaltree with birdtree, etc. depending on dataset being analysed
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#--------------------------------------#
# Cover * FSCI interaction
#--------------------------------------#
# loops through each of 100 trees selected randomly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  # the model parameterization d.forcov.allr*d.fsci.allr by default specifies both main and two-way interaction effects
  cover.fsci <- phyloglm(rl.cat ~ d.forcov.allr*d.fsci.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover.fsci <- cover.fsci$aic # AIC
  logl.cover.fsci <- cover.fsci$logLik # log likelihood
  plogl.cover.fsci <- cover.fsci$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.cover.fsci, logl.cover.fsci, plogl.cover.fsci) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover.fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover.fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover.fsci <- cover.fsci$X
  yvalues.cover.fsci <- cover.fsci$y
  lin.pred.cover.fsci <- cover.fsci$linear.predictors
  fitted.val.cover.fsci <- cover.fsci$fitted.values
  residuals.cover.fsci <- cover.fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover.fsci, yvalues.cover.fsci, lin.pred.cover.fsci, fitted.val.cover.fsci, residuals.cover.fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/cover.fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef.confint, file = "Mammals/RL/cover.fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha.confint, file = "Mammals/RL/cover.fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/RL/cover.fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop

#--------------------------------------#
# Cover * FSII interaction
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover.fsii <- phyloglm(rl.cat ~ d.forcov.allr*d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover.fsii <- cover.fsii$aic
  logl.cover.fsii <- cover.fsii$logLik
  plogl.cover.fsii <- cover.fsii$penlogLik
  mod.sel <- cbind(aic.cover.fsii, logl.cover.fsii, plogl.cover.fsii)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover.fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover.fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover.fsii <- cover.fsii$X
  yvalues.cover.fsii <- cover.fsii$y
  lin.pred.cover.fsii <- cover.fsii$linear.predictors
  fitted.val.cover.fsii <- cover.fsii$fitted.values
  residuals.cover.fsii <- cover.fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover.fsii, yvalues.cover.fsii, lin.pred.cover.fsii, fitted.val.cover.fsii, residuals.cover.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/cover.fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/cover.fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/cover.fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/cover.fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------#
# FSCI * FSII interaction
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsci.fsii <- phyloglm(rl.cat ~ d.fsci.allr*d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Data/variables to extract from model
  # Model selection
  aic.fsci.fsii <- fsci.fsii$aic
  logl.fsci.fsii <- fsci.fsii$logLik
  plogl.fsci.fsii <- fsci.fsii$penlogLik
  mod.sel <- cbind(aic.fsci.fsii, logl.fsci.fsii, plogl.fsci.fsii)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(fsci.fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsci.fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsci.fsii <- fsci.fsii$X
  yvalues.fsci.fsii <- fsci.fsii$y
  lin.pred.fsci.fsii <- fsci.fsii$linear.predictors
  fitted.val.fsci.fsii <- fsci.fsii$fitted.values
  residuals.fsci.fsii <- fsci.fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsci.fsii, yvalues.fsci.fsii, lin.pred.fsci.fsii, fitted.val.fsci.fsii, residuals.fsci.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/fsci.fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/RL/fsci.fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/RL/fsci.fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/fsci.fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Population Trend as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

# Read in the Excel workbook with the data as above before running the lines below

#-----------------------------------#
# Recode Population Trend as numeric
#-----------------------------------#
mammals[,5] <- ifelse(mammals[,5] == "Decreasing", 1,
                      ifelse(mammals[,5] == "Stable or Increasing", 0,
                             ifelse(mammals[,5] == "Unknown", 2, NA)))


mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests

# These two lines allow subsetting of species by htf dependency, use one at a time depending on whether rainforest obligate or associated species are being analysed
mammals5 <- subset(mammals4, htf.dependency == "Exclusive") # keep only humid tropical forest obligate species 
#mammals5 <- subset(mammals4, htf.dependency == "Optional") # keep only humid tropical forest associated species

mammals6 <- subset(mammals5, pop.trend < 2) # remove unknown population trend species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#--------------------------------------#
# Cover * FSCI interaction
#--------------------------------------#
# loops through each of 100 trees selected randonly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover.fsci <- phyloglm(pop.trend ~ d.forcov.allr*d.fsci.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover.fsci <- cover.fsci$aic # AIC
  logl.cover.fsci <- cover.fsci$logLik # log likelihood
  plogl.cover.fsci <- cover.fsci$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.cover.fsci, logl.cover.fsci, plogl.cover.fsci) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover.fsci)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover.fsci), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover.fsci <- cover.fsci$X
  yvalues.cover.fsci <- cover.fsci$y
  lin.pred.cover.fsci <- cover.fsci$linear.predictors
  fitted.val.cover.fsci <- cover.fsci$fitted.values
  residuals.cover.fsci <- cover.fsci$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover.fsci, yvalues.cover.fsci, lin.pred.cover.fsci, fitted.val.cover.fsci, residuals.cover.fsci)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/cover.fsci_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef.confint, file = "Mammals/PT/cover.fsci_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha.confint, file = "Mammals/PT/cover.fsci_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/PT/cover.fsci_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop

#--------------------------------------#
# Cover * FSII interaction
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  cover.fsii <- phyloglm(pop.trend ~ d.forcov.allr*d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.cover.fsii <- cover.fsii$aic
  logl.cover.fsii <- cover.fsii$logLik
  plogl.cover.fsii <- cover.fsii$penlogLik
  mod.sel <- cbind(aic.cover.fsii, logl.cover.fsii, plogl.cover.fsii)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(cover.fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(cover.fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha.confint <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.cover.fsii <- cover.fsii$X
  yvalues.cover.fsii <- cover.fsii$y
  lin.pred.cover.fsii <- cover.fsii$linear.predictors
  fitted.val.cover.fsii <- cover.fsii$fitted.values
  residuals.cover.fsii <- cover.fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.cover.fsii, yvalues.cover.fsii, lin.pred.cover.fsii, fitted.val.cover.fsii, residuals.cover.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/cover.fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/cover.fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/cover.fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/cover.fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#--------------------------------------#
# FSCI * FSII interaction
#--------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  fsci.fsii <- phyloglm(pop.trend ~ d.fsci.allr*d.fsii.allr, data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Data/variables to extract from model
  # Model selection
  aic.fsci.fsii <- fsci.fsii$aic
  logl.fsci.fsii <- fsci.fsii$logLik
  plogl.fsci.fsii <- fsci.fsii$penlogLik
  mod.sel <- cbind(aic.fsci.fsii, logl.fsci.fsii, plogl.fsci.fsii)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(fsci.fsii)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsci.fsii), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:4], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:4], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef.confint <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,5], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,5], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsci.fsii <- fsci.fsii$X
  yvalues.fsci.fsii <- fsci.fsii$y
  lin.pred.fsci.fsii <- fsci.fsii$linear.predictors
  fitted.val.fsci.fsii <- fsci.fsii$fitted.values
  residuals.fsci.fsii <- fsci.fsii$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsci.fsii, yvalues.fsci.fsii, lin.pred.fsci.fsii, fitted.val.fsci.fsii, residuals.fsci.fsii)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/fsci.fsii_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef.confint, file = "Mammals/PT/fsci.fsii_allcoef.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha.confint, file = "Mammals/PT/fsci.fsii_allalpha.confint.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/fsci.fsii_allsp.linpred.fittedval.residuals.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop
