library(readxl)
library(phylolm)
library(future)

# Set working directory or create project folder before running code

mammals.HTF <- read_excel("HTF_Mammals.xlsx", sheet = "HTF_Mammals") # all datasets are formatted identically, replace names (e.g., "HTF_Birds.xlsx", sheet = "HTF_Birds") to analyse other taxonomic groups

# Create dataframe containing variables for analyses, see README file for description of each variable, replace Species_MammalTree with Species_BirdTree, etc depending on dataset being analysed
mammals <- data.frame(species=mammals.HTF$Species_MammalTree, order=mammals.HTF$Order, family=mammals.HTF$Family,
                      rl.cat=mammals.HTF$Red_List_Category2, pop.trend=mammals.HTF$Population_Trend2,
                      realm=mammals.HTF$Realm2, gr.1realm=mammals.HTF$`Occurs_In_>1_Realm`,
                      tfspecies=as.numeric(mammals.HTF$'1_Tropical_Forest'), htf.dependency=mammals.HTF$HTF_Dependency,
                      d.forcov.allr=scale(mammals.HTF$Rel_Diff_Forest_NonForest_AllRealms), d.fsci.allr=scale(mammals.HTF$Rel_Diff_H_L_FSCI_AllRealms), d.fsii.allr=scale(mammals.HTF$Rel_Diff_H_L_FSII_AllRealms))

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Red List or threatened status as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

#----------------------------------#
# Recode Red List status as numeric
#----------------------------------#
mammals[,4] <- ifelse(mammals[,4] == "Threatened", 1,
                      ifelse(mammals[,4] == "Not Threatened", 0,
                             ifelse(mammals[,4] == "Data Deficient", 2, NA)))

mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests
mammals5 <- subset(mammals4, gr.1realm=="No") # keep only species that occur in 1 realm, filter based on whether or not a species occurs in > 1 realm

# htf dependency is specified as a categorical covariate in realm models and is therefore not subset

mammals6 <- subset(mammals5, rl.cat < 2) # remove Data Deficient or DD species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species, replace mammaltree with birdtree, etc. depending on dataset being analysed
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#-----------------------------------------------------------------------------------------------------------#
# Model 1 (baseline model): realm as a categorical predictor variable 
#-----------------------------------------------------------------------------------------------------------#
# loops through each of 100 trees selected randomly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  realm <- phyloglm(rl.cat ~ factor(realm) + factor(htf.dependency), data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.realm <- realm$aic # AIC
  logl.realm <- realm$logLik # log likelihood
  plogl.realm <- realm$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.realm, logl.realm, plogl.realm) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(realm)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(realm), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:5], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:5], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,6], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,6], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.realm <- realm$X
  yvalues.realm <- realm$y
  lin.pred.realm <- realm$linear.predictors
  fitted.val.realm <- realm$fitted.values
  residuals.realm <- realm$residuals
  all.sp.specific <- data.frame(species.sp.coef.realm, yvalues.realm, lin.pred.realm, fitted.val.realm, residuals.realm)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/realm_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef, file = "Mammals/RL/realm_allcoef.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha, file = "Mammals/RL/realm_allalpha.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/RL/realm_allsp.lp.ft.rs.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop


#-----------------------------------------------------------------------------------------------------------#
# Model 2: two way interactions between fsii*realm and cover*realm
#-----------------------------------------------------------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  # the model parameterization shown below by default specifies both main effects of each variable and two-way interaction effects between forest integrity and realm as well as forest cover and realm
  fsii.cover.realm <- phyloglm(rl.cat ~ d.forcov.allr*factor(realm) + d.fsii.allr*factor(realm) + factor(htf.dependency), data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsii.cover.realm <- fsii.cover.realm$aic
  logl.fsii.cover.realm <- fsii.cover.realm$logLik
  plogl.fsii.cover.realm <- fsii.cover.realm$penlogLik
  mod.sel <- cbind(aic.fsii.cover.realm, logl.fsii.cover.realm, plogl.fsii.cover.realm)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(fsii.cover.realm)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsii.cover.realm), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:13], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:13], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,14], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,14], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsii.cover.realm <- fsii.cover.realm$X
  yvalues.fsii.cover.realm <- fsii.cover.realm$y
  lin.pred.fsii.cover.realm <- fsii.cover.realm$linear.predictors
  fitted.val.fsii.cover.realm <- fsii.cover.realm$fitted.values
  residuals.fsii.cover.realm <- fsii.cover.realm$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsii.cover.realm, yvalues.fsii.cover.realm, lin.pred.fsii.cover.realm, fitted.val.fsii.cover.realm, residuals.fsii.cover.realm)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/RL/fsii.cover.realm_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef, file = "Mammals/RL/fsii.cover.realm_allcoef.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha, file = "Mammals/RL/fsii.cover.realm_allalpha.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/RL/fsii.cover.realm_allsp.lp.ft.rs.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# For analyses of IUCN Population Trend as the response variable
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

# Read in the Excel workbook with the data as above before running the lines below

#-----------------------------------#
# Recode Population Trend as numeric
#-----------------------------------#
mammals[,5] <- ifelse(mammals[,5] == "Decreasing", 1,
                      ifelse(mammals[,5] == "Stable or Increasing", 0,
                             ifelse(mammals[,5] == "Unknown", 2, NA)))

mammals2 <- subset(mammals, species !="NA") # subset species to remove NAs i.e. species that are not in the mammal phylogenetic tree
mammals2$species <- sub(" ", "_", mammals2$species) # replace space between genus and species with underscore to match formatting in phylogenetic tree
mammals3 <- mammals2[!duplicated(mammals2$species), ] # remove duplicated rows for species i.e. species that occur in > 1 realm
mammals4 <- subset(mammals3, tfspecies > 0) # remove remaining species not associated with tropical forests
mammals5 <- subset(mammals4, gr.1realm=="No") # keep only species that occur in 1 realm, filter based on whether or not a species occurs in > 1 realm

# htf dependency is specified as a categorical covariate in realm models and is therefore not subset

mammals6 <- subset(mammals5, pop.trend < 2) # remove Data Deficient or DD species
rownames(mammals6) <- mammals6$species # phyloglm function requires row names of data to be species names

# read in the phylogenetic tree after downloading from https://vertlife.org/
mammaltree10k <- read.nexus("Mammals/MammalTree_3351_HTF_Species/output.nex") # 10000 trees, from node-dated completed trees all 5911 species
class(mammaltree10k)
length(mammaltree10k)

# Randomly sample 100 trees from the posterior sample of 10000 
random.mammaltree <- sample(mammaltree10k, size=100)
class(random.mammaltree)
length(random.mammaltree)

# set up parallel processing via futures package
multisession(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = 24, gc = FALSE, earlySignal = FALSE, label = NULL)
plan(multisession)

#-----------------------------------------------------------------------------------------------------------#
# Model 1 (baseline model): realm as a categorical predictor variable 
#-----------------------------------------------------------------------------------------------------------#
# loops through each of 100 trees selected randomly above, running the model and outputting results to CSVs
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  realm <- phyloglm(pop.trend ~ factor(realm) + factor(htf.dependency), data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.realm <- realm$aic # AIC
  logl.realm <- realm$logLik # log likelihood
  plogl.realm <- realm$penlogLik # penalized log likelihood
  mod.sel <- cbind(aic.realm, logl.realm, plogl.realm) # columns are created in the order specified here
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(realm)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(realm), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:5], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:5], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,6], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,6], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.realm <- realm$X
  yvalues.realm <- realm$y
  lin.pred.realm <- realm$linear.predictors
  fitted.val.realm <- realm$fitted.values
  residuals.realm <- realm$residuals
  all.sp.specific <- data.frame(species.sp.coef.realm, yvalues.realm, lin.pred.realm, fitted.val.realm, residuals.realm)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/realm_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # model selection
  write.table(allcoef, file = "Mammals/PT/realm_allcoef.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # estimated coefficients and associated uncertainty (95% CIs), p-values
  write.table(allalpha, file = "Mammals/PT/realm_allalpha.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # alpha values - phylogenetic correlation
  write.table(all.sp.specific, file = "Mammals/PT/realm_allsp.lp.ft.rs.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE) # linear predictors, fitted values, residuals, etc
} # End of loop


#-----------------------------------------------------------------------------------------------------------#
# Model 2: two way interactions between fsii*realm and cover*realm
#-----------------------------------------------------------------------------------------------------------#
for(i in 1:length(random.mammaltree)){
  
  # prune tree to retain only species in the list being analyzed, for each iteration of the loop
  pruned.trees <- drop.tip(random.mammaltree[[i]], setdiff(random.mammaltree[[i]]$tip.label, mammals6$species))
  
  # the model parameterization shown below by default specifies both main effects of each variable and two-way interaction effects between forest integrity and realm as well as forest cover and realm
  fsii.cover.realm <- phyloglm(pop.trend ~ d.forcov.allr*factor(realm) + d.fsii.allr*factor(realm) + factor(htf.dependency), data=mammals6, phy=pruned.trees, method="logistic_MPLE", btol=20, log.alpha.bound=4, boot=2000)
  
  # Estimated parameters to extract from models at each iteration
  # Model selection
  aic.fsii.cover.realm <- fsii.cover.realm$aic
  logl.fsii.cover.realm <- fsii.cover.realm$logLik
  plogl.fsii.cover.realm <- fsii.cover.realm$penlogLik
  mod.sel <- cbind(aic.fsii.cover.realm, logl.fsii.cover.realm, plogl.fsii.cover.realm)
  
  # estimated coefficients and associated uncertainty (95% CIs), p-values
  s <- summary(fsii.cover.realm)
  coef.est <- round(s$coefficients, digits=10) # extracts entire summary in a table
  confint.est <- round(confint(fsii.cover.realm), digits=10) # 95% CIs estimated with the confint function
  boot.coef.est <- round(s$bootmean[1:13], digits=10) # bootstrapped coefficient estimates analogous to estimated coefficients
  boot.coef.sd <- round(s$bootsd[1:13], digits=10) # bootstrapped standard deviation of estimated coefficients
  allcoef <- cbind(coef.est, confint.est, boot.coef.est, boot.coef.sd)
  
  # alpha values - phylogenetic correlation
  alpha.est <- round(s$alpha, digits=10) # estimated alpha
  alpha.boot <- round(s$bootmeanAlog, digits=10) # bootstrapped estimated alpha log scale
  alpha.boot.backtr <- round(exp(s$bootmeanAlog), digits=10) # bootstrapped estimated alpha backtransformed
  alpha.boot.2.5 <- round(s$bootconfint95[1,14], digits=10) # bootstrapped lower 95% CI alpha estimate
  alpha.boot.97.5 <- round(s$bootconfint95[2,14], digits=10) # bootstrapped upped 95% CI alpha estimate
  allalpha <- cbind(alpha.est, alpha.boot, alpha.boot.backtr, alpha.boot.2.5, alpha.boot.97.5)
  
  # linear predictors, fitted values, residuals, etc
  species.sp.coef.fsii.cover.realm <- fsii.cover.realm$X
  yvalues.fsii.cover.realm <- fsii.cover.realm$y
  lin.pred.fsii.cover.realm <- fsii.cover.realm$linear.predictors
  fitted.val.fsii.cover.realm <- fsii.cover.realm$fitted.values
  residuals.fsii.cover.realm <- fsii.cover.realm$residuals
  all.sp.specific <- data.frame(species.sp.coef.fsii.cover.realm, yvalues.fsii.cover.realm, lin.pred.fsii.cover.realm, fitted.val.fsii.cover.realm, residuals.fsii.cover.realm)
  
  # write all outputs to CSVs, change working directory name as necessary
  write.table(mod.sel, file = "Mammals/PT/fsii.cover.realm_mod.sel.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allcoef, file = "Mammals/PT/fsii.cover.realm_allcoef.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(allalpha, file = "Mammals/PT/fsii.cover.realm_allalpha.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
  write.table(all.sp.specific, file = "Mammals/PT/fsii.cover.realm_allsp.lp.ft.rs.csv", sep = ",", col.names = FALSE, row.names = TRUE, append = TRUE)
} # End of loop
