'''
April 2020

Script for tabulating the area of forest integrity and human footprint raster datasets within the ranges of
vertebrate species by humid tropical forest type and biogeographic realm.

The input is a GDB with species ranges containing forest type and realm attributes, organized into feature datasets.
The output is a csv file with the humid tropical forest area under each of multiple SCI/FSII/HFP classes per species for a specified realm.

This script uses arcpy from ArcGIS Pro for spatial analysis.

Each feature dataset (FDS) holds multiple polygons of species ranges.

All Feature Class (Range map polygons) and Raster (SCI/FSII/HFP)
Datasets use the projection World Mollweide.

Each Feature Class for each taxonomic group was imported, checked, and 
geometries were repaired beforehand



Script developed by Jose Aragon-Osejo (jose.luis.aragon.ec@gmail.com)
and Rajeev Pillay (rajeev.p20a@gmail.com)

Natural Resources and Environmental Studies Institute

University of Northern British Columbia, Prince George, Canada

'''


import arcpy
from arcpy.sa import *
import pandas as pd
from pandas import DataFrame


def feature_class_to_pandas_data_frame(feature_class, field_list):
    """
    Load data into a Pandas Data Frame for subsequent analysis.
    :param feature_class: Input ArcGIS Feature Class.
    :param field_list: Fields for input.
    :return: Pandas DataFrame object.
    """
    return DataFrame(
        arcpy.da.FeatureClassToNumPyArray(
            in_table=feature_class,
            field_names=field_list,
            skip_nulls=False,
            null_value=-99999
        )
    )

###############################################################################
# Change parameters here:

#Paths:

arcpy.env.overwriteOutput = True
arcpy.CheckOutExtension("Spatial")
main_folder = r'C:\Users\pillay\Desktop\Mammals\\'
GDB = main_folder + r'Mammals.gdb\\' # change depending on the taxonomic group being analyzed
output_folder = r"C:\Users\pillay\Desktop\Mammals\Tables\\" # create a Tables folder within the working directory before running the script. CSVs will be saved here.
raster = GDB + 'HTF_FSCI2018_1KBilinear' # download from citations provided in the paper and change depending on the raster dataset being analysed
raster_field = 'VALUE'
species_FDSs = [ # Feature datasets containing range map polygon layers from IUCN and other sources, for computing efficiency the original IUCN shapefiles were subdivided into feature datasets containing 1000 species range maps
        'FDS3_Mammals01_ecor',
        'FDS2_Mammals02',
        'FDS2_Mammals03',
        'FDS2_Mammals04',
        'FDS2_Mammals05',
        'FDS2_Mammals06',
        ]
species_field = 'binomial'
table = GDB + r'aux_table'
HTF_field = 'Forest_Type'
expr1 = HTF_field + " = 'HTF'" # tabulate area for only humid tropical forest biome
REALM_field = 'REALM'
expr2 = REALM_field + " = 'Afrotropic'" # tabulate area for HTF only in Afrotropic realm, change depending on realm being analysed

errors = []

for species_FDS in species_FDSs:
    print('Processing ' + species_FDS)
    arcpy.env.workspace = GDB + species_FDS
    featureclasses = arcpy.ListFeatureClasses()        
    df = pd.DataFrame({'A' : []})

    for sp in featureclasses:
        desc = arcpy.Describe(sp)
        sp_name = desc.name
        print('   Tabulating {}'.format(sp_name))
        try:
            arcpy.MakeFeatureLayer_management(sp,"sp_lyr")
            arcpy.SelectLayerByAttribute_management("sp_lyr", "NEW_SELECTION", expr1)
            arcpy.SelectLayerByAttribute_management("sp_lyr", "SUBSET_SELECTION", expr2)
            if arcpy.management.GetCount("sp_lyr")[0] != "0": 
                TabulateArea("sp_lyr", species_field, raster, raster_field, table,'')
                field_names = [f.name for f in arcpy.ListFields(table)]
                field_names.pop(0)
                if not df.empty:
                    df_inFeatures = feature_class_to_pandas_data_frame(table,field_names)
                    df = pd.concat([df,df_inFeatures],sort=True)
                else:
                    df = feature_class_to_pandas_data_frame(table,field_names)
        except Exception as e:    
            print('!!! {}'.format(e))
            errors.append({e:sp_name})

    df.to_csv(output_folder + species_FDS + '.csv',index=False, na_rep=0)
print('---------------------------')
if errors:
    print('   Species with errors: {}'.format(errors))
else:
    print('   No errors found during processing')
print()
print('---Fin---')
